from pathlib import Path

from django.core.management.base import BaseCommand

from drf_to_mkdoc.conf.settings import drf_to_mkdoc_settings
from drf_to_mkdoc.utils.common import get_schema, load_model_json_data
from drf_to_mkdoc.utils.endpoint_detail_generator import (
    generate_endpoint_files,
    parse_endpoints_from_schema,
)
from drf_to_mkdoc.utils.endpoint_list_generator import create_endpoints_index
from drf_to_mkdoc.utils.model_detail_generator import generate_model_docs
from drf_to_mkdoc.utils.model_list_generator import create_models_index


class Command(BaseCommand):
    help = "Generate complete API documentation (models + endpoints + navigation)"

    def add_arguments(self, parser):
        parser.add_argument(
            "--endpoints-only",
            action="store_true",
            help="Generate only endpoint documentation",
        )
        parser.add_argument(
            "--models-only",
            action="store_true",
            help="Generate only model documentation",
        )

    def handle(self, *args, **options):
        self.stdout.write(self.style.SUCCESS("🚀 Starting documentation generation..."))

        generate_models = not options["endpoints_only"]
        generate_endpoints = not options["models_only"]
        if not generate_models and not generate_endpoints:
            self.stdout.write(
                self.style.ERROR(
                    "❌ No outputs selected: --models-only and --endpoints-only cannot be used together"
                )
            )
            return
        docs_dir = self._setup_docs_directory()
        models_data = self._load_models_data() if generate_models else {}
        schema_data = self._load_schema_data() if generate_endpoints else {}

        if generate_models and models_data:
            self._generate_models_documentation(models_data, docs_dir)

        if generate_endpoints and schema_data:
            self._generate_endpoints_documentation(schema_data, docs_dir)

        self.stdout.write(self.style.SUCCESS("✅ Documentation generation complete!"))

    def _setup_docs_directory(self):
        docs_dir = Path(drf_to_mkdoc_settings.DOCS_DIR)
        docs_dir.mkdir(parents=True, exist_ok=True)
        return docs_dir

    def _load_models_data(self):
        json_data = load_model_json_data()
        models_data = json_data.get("models", {}) if json_data else {}

        if not models_data:
            self.stdout.write(self.style.WARNING("⚠️  No model data found"))

        return models_data

    def _load_schema_data(self):
        try:
            schema = get_schema()
        except Exception as e:
            self.stdout.write(self.style.ERROR(f"❌ Failed to load OpenAPI schema: {e}"))
            return {}
        if not schema:
            self.stdout.write(self.style.ERROR("❌ Failed to load OpenAPI schema"))
            return {}

        paths = schema.get("paths", {})
        components = schema.get("components", {})

        self.stdout.write(f"📊 Loaded {len(paths)} API paths")

        return {"paths": paths, "components": components}

    def _generate_models_documentation(self, models_data, docs_dir):
        self.stdout.write("📋 Generating model documentation...")

        try:
            generate_model_docs(models_data)
            create_models_index(models_data, docs_dir)
            self.stdout.write(self.style.SUCCESS("✅ Model documentation generated"))
        except Exception as e:
            self.stdout.write(self.style.WARNING(f"⚠️  Failed to generate model docs: {e}"))
            if hasattr(self, "_generating_all"):
                self.stdout.write(self.style.WARNING("Continuing with endpoint generation..."))
            raise

    def _generate_endpoints_documentation(self, schema_data, docs_dir):
        self.stdout.write("🔗 Generating endpoint documentation...")

        paths = schema_data["paths"]
        components = schema_data["components"]

        endpoints_by_app = parse_endpoints_from_schema(paths)
        total_endpoints = generate_endpoint_files(endpoints_by_app, components)
        create_endpoints_index(endpoints_by_app, docs_dir)

        self.stdout.write(
            self.style.SUCCESS(
                f"✅ Generated {total_endpoints} endpoint files with Django view introspection"
            )
        )



from typing import Any


def _md_query_parameters_format_error_section(query_params: dict[str, Any]) -> str:
    if "error" in query_params:
        return f"**Error:** {query_params['error']}\n\n"
    return ""


def _md_query_parameters_format_search_section(query_params: dict[str, Any]) -> str:
    search_fields = query_params.get("search_fields", [])
    if search_fields:
        content = "### Search Parameters\n\n"
        for field in search_fields:
            content += f"- `{field}`\n"
        content += "\n"
        return content
    return ""


def _md_query_parameters_format_filter_section(query_params: dict[str, Any]) -> str:
    filter_fields = query_params.get("filter_fields", [])
    if filter_fields:
        content = "### Filter Parameters\n\n"
        for field in filter_fields:
            content += f"- `{field}`\n"
        content += "\n"
        return content
    return ""


def _md_query_parameters_format_ordering_section(query_params: dict[str, Any]) -> str:
    ordering_fields = query_params.get("ordering_fields", [])
    if ordering_fields:
        content = "### Ordering Parameters\n\n"
        for field in ordering_fields:
            content += f"- `{field}`\n"
        content += "\n"
        return content
    return ""


def _md_query_parameters_format_pagination_section(query_params: dict[str, Any]) -> str:
    pagination_fields = query_params.get("pagination_fields", [])
    if pagination_fields:
        content = "### Pagination Parameters\n\n"
        for field in pagination_fields:
            content += f"- `{field}`\n"
        content += "\n"
        return content
    return ""


def _md_query_parameters_format_filter_backends_section(query_params: dict[str, Any]) -> str:
    filter_backends = query_params.get("filter_backends", [])
    if filter_backends:
        return f"**Filter Backends:** {', '.join(filter_backends)}\n\n"
    return ""


def generate_query_parameters_md(query_params: dict[str, Any]) -> str:
    """Format query parameters into markdown documentation"""
    if "error" in query_params:
        return _md_query_parameters_format_error_section(query_params)

    content = ""
    content += _md_query_parameters_format_search_section(query_params)
    content += _md_query_parameters_format_filter_section(query_params)
    content += _md_query_parameters_format_ordering_section(query_params)
    content += _md_query_parameters_format_pagination_section(query_params)
    content += _md_query_parameters_format_filter_backends_section(query_params)
    return content



from django.conf import settings

from drf_to_mkdoc.conf.defaults import DEFAULTS


class DRFToMkDocSettings:
    required_settings = ["DJANGO_APPS"]
    project_settings = {"PROJECT_NAME": "drf-to-mkdoc"}

    def __init__(self, user_settings_key="DRF_TO_MKDOC", defaults=None):
        self.user_settings_key = user_settings_key
        self._user_settings = getattr(settings, user_settings_key, {})
        self.defaults = defaults or {}

    def get(self, key):
        if key not in self.defaults:
            if key in self.project_settings:
                return self.project_settings[key]
            raise AttributeError(f"Invalid DRF_TO_MKDOC setting: '{key}'")

        value = self._user_settings.get(key, self.defaults[key])

        if value is None and key in self.required_settings:
            raise ValueError(
                f"DRF_TO_MKDOC setting '{key}' is required but not configured. "
                f"Please add it to your Django settings under {self.user_settings_key}."
            )

        return value

    def __getattr__(self, key):
        return self.get(key)

    def validate_required_settings(self):
        missing_settings = []

        for setting in self.required_settings:
            try:
                self.get(setting)
            except ValueError:
                missing_settings.append(setting)

        if missing_settings:
            raise ValueError(
                f"Missing required settings: {', '.join(missing_settings)}. "
                f"Please configure these in your Django settings under {self.user_settings_key}."
            )


drf_to_mkdoc_settings = DRFToMkDocSettings(defaults=DEFAULTS)



from pathlib import Path
from typing import Any

from django.templatetags.static import static

from drf_to_mkdoc.conf.settings import drf_to_mkdoc_settings
from drf_to_mkdoc.utils.common import get_app_descriptions


def create_models_index(models_data: dict[str, Any], docs_dir: Path) -> None:
    """Create the main models index page that lists all models organized by app."""
    models_by_app = {}
    for model_name, model_info in models_data.items():
        app_name = model_info.get("app_label", model_name.split(".")[0])
        class_name = model_info.get("name", model_name.split(".")[-1])
        if app_name not in models_by_app:
            models_by_app[app_name] = []
        models_by_app[app_name].append((class_name, model_name, model_info))

    stylesheets = [
        "stylesheets/models/variables.css",
        "stylesheets/models/base.css",
        "stylesheets/models/model-cards.css",
        "stylesheets/models/responsive.css",
        "stylesheets/models/animations.css",
    ]
    prefix_path = f"{drf_to_mkdoc_settings.PROJECT_NAME}/"
    css_links = "\n".join(
        f'<link rel="stylesheet" href="{static(prefix_path + path)}">' for path in stylesheets
    )
    content = f"""# Django Models

This section contains documentation for all Django models in the system, organized by Django application.

<!-- inject CSS directly -->
{css_links}

<div class="models-container">
"""

    app_descriptions = get_app_descriptions()

    for app_name in sorted(models_by_app.keys()):
        app_desc = app_descriptions.get(app_name, f"{app_name.title()} application models")
        content += f'<div class="app-header">{app_name.title()} App</div>\n'
        content += f'<div class="app-description">{app_desc}</div>\n\n'

        content += '<div class="model-cards">\n'

        for class_name, _model_name, _model_info in sorted(models_by_app[app_name]):
            content += f"""
            <a href="{app_name}/{class_name.lower()}/"
             class="model-card">{class_name}</a>\n
"""

        content += "</div>\n\n"

    content += """</div>


Each model page contains detailed field documentation, method signatures, and relationships to other models."""

    models_index_path = docs_dir / "models" / "index.md"
    models_index_path.parent.mkdir(parents=True, exist_ok=True)

    with models_index_path.open("w", encoding="utf-8") as f:
        f.write(content)



from typing import Any

from django.templatetags.static import static

from drf_to_mkdoc.conf.settings import drf_to_mkdoc_settings
from drf_to_mkdoc.utils.common import get_model_description, write_file


def generate_model_docs(models_data: dict[str, Any]) -> None:
    """Generate model documentation from JSON data"""
    for model_name, model_info in models_data.items():
        app_name = model_info.get("app_label", model_name.split(".")[0])
        class_name = model_info.get("name", model_name.split(".")[-1])

        # Create the model page content
        content = create_model_page(model_info)

        # Write the file in app subdirectory
        file_path = f"models/{app_name}/{class_name.lower()}.md"
        write_file(file_path, content)


def render_column_fields_table(fields: dict[str, Any]) -> str:
    """Render the fields table for a model."""
    content = "## Fields\n\n"
    content += "| Field | Type | Description | Extra |\n"
    content += "|-------|------|-------------|-------|\n"

    for field_name, field_info in fields.items():
        field_type = field_info.get("type", "Unknown")
        verbose_name = field_info.get("verbose_name", field_name)
        help_text = field_info.get("help_text", "")

        display_name = field_name
        if field_type in ["ForeignKey", "OneToOneField"]:
            display_name = f"{field_name}_id"

        extra_info = []
        if field_info.get("null"):
            extra_info.append("null=True")
        if field_info.get("blank"):
            extra_info.append("blank=True")
        if field_info.get("unique"):
            extra_info.append("unique=True")
        if field_info.get("primary_key"):
            extra_info.append("primary_key=True")
        if field_info.get("default"):
            extra_info.append(f"default={field_info['default']}")

        field_specific = field_info.get("field_specific", {})
        for key, value in field_specific.items():
            if key not in ["related_name", "related_query_name", "to"]:
                extra_info.append(f"{key}={value}")

        extra_str = ", ".join(extra_info) if extra_info else ""
        description_str = help_text or verbose_name

        content += f"| `{display_name}` | {field_type} | {description_str} | {extra_str} |\n"

    return content


def render_choices_tables(fields: dict[str, Any]) -> str:
    """Render choice tables for fields with choices."""
    choice_tables = []

    for field_name, field_info in fields.items():
        choices = field_info.get("choices")
        if choices:
            table = f"### {field_name} Choices\n\n"
            table += "| Label | Value |\n"
            table += "|-------|--------|\n"
            for choice in choices:
                table += f"| {choice['display']} | `{choice['value']}` |\n"
            table += "\n"
            choice_tables.append(table)

    if choice_tables:
        return "## Choices\n\n" + "\n".join(choice_tables)
    return ""


def create_model_page(model_info: dict[str, Any]) -> str:
    """Create a model documentation page from model info"""
    name = model_info.get("name", "Unknown")
    app_label = model_info.get("app_label", "unknown")
    table_name = model_info.get("table_name", "")
    description = get_model_description(name)

    content = _create_model_header(name, app_label, table_name, description)
    content += _add_fields_section(model_info)
    content += _add_relationships_section(model_info)
    content += _add_methods_section(model_info)
    content += _add_meta_options_section(model_info)

    return content


def _create_model_header(name: str, app_label: str, table_name: str, description: str) -> str:
    """Create the header section of the model documentation."""
    stylesheets = [
        "stylesheets/models/variables.css",
        "stylesheets/models/base.css",
        "stylesheets/models/model-tables.css",
        "stylesheets/models/responsive.css",
    ]
    prefix_path = f"{drf_to_mkdoc_settings.PROJECT_NAME}/"
    css_links = "\n".join(
        f'<link rel="stylesheet" href="{static(prefix_path + path)}">' for path in stylesheets
    )
    return f"""# {name}

<!-- inject CSS directly -->
{css_links}

**App:** {app_label}
**Table:** `{table_name}`

## Description

{description}

"""


def _add_fields_section(model_info: dict[str, Any]) -> str:
    """Add the fields section to the model documentation."""
    column_fields = model_info.get("column_fields", {})
    if not column_fields:
        return ""

    content = ""

    column_fields_content = render_column_fields_table(column_fields)
    if column_fields_content:
        content += column_fields_content
        content += "\n"

    choices_content = render_choices_tables(column_fields)
    if choices_content:
        content += choices_content
        content += "\n"

    return content


def _add_relationships_section(model_info: dict[str, Any]) -> str:
    """Add the relationships section to the model documentation."""
    relationship_fields = model_info.get("relationships", {})
    if not relationship_fields:
        return ""

    content = "## Relationships\n\n"
    content += "| Field | Type | Related Model |\n"
    content += "|-------|------|---------------|\n"

    content += _render_relationship_fields(relationship_fields)
    content += _render_relationships_from_section(relationship_fields)
    content += "\n"

    return content


def _render_relationship_fields(relationship_fields: dict[str, Any]) -> str:
    """Render relationship fields from the fields section."""
    content = ""
    for field_name, field_info in relationship_fields.items():
        field_type = field_info.get("type", "Unknown")
        field_specific = field_info.get("field_specific", {})
        to_model = field_specific.get("to", "")

        if to_model:
            model_link = _create_model_link(to_model)
            content += f"| `{field_name}` | {field_type} | {model_link}|\n"

    return content


def _render_relationships_from_section(relationships: dict[str, Any]) -> str:
    """Render relationships from the relationships section."""
    content = ""
    for rel_name, rel_info in relationships.items():
        rel_type = rel_info.get("type", "Unknown")
        related_model_full = rel_info.get("related_model", "")

        if related_model_full and "." in related_model_full:
            related_app, related_model = related_model_full.split(".", 1)
            model_link = f"[{related_model}](../../{related_app}/{related_model.lower()}/)"
        else:
            model_link = related_model_full

        content += f"| `{rel_name}` | {rel_type} | {model_link} | \n"

    return content


def _create_model_link(to_model: str) -> str:
    """Create a link to a related model."""
    if "." in to_model:
        related_app, related_model = to_model.split(".", 1)
        return f"[{related_model}](../{related_app}/{related_model.lower()}/)"
    return f"[{to_model}]({to_model.lower()}/)"


def _add_methods_section(model_info: dict[str, Any]) -> str:
    """Add the methods section to the model documentation."""
    methods = model_info.get("methods", [])
    if not methods:
        return ""

    content = "## Methods\n\n"
    for method in methods:
        method_name = method.get("name", "")
        docstring = method.get("docstring", "")

        content += f"### `{method_name}()`\n\n"
        if docstring:
            content += f"{docstring}\n\n"
        else:
            content += "No documentation available.\n\n"

    return content


def _add_meta_options_section(model_info: dict[str, Any]) -> str:
    """Add the meta options section to the model documentation."""
    meta_options = model_info.get("meta_options", {})
    if not meta_options:
        return ""

    content = "## Meta Options\n\n"
    for option, value in meta_options.items():
        content += f"- **{option}:** {value}\n"
    content += "\n"

    return content



from pathlib import Path
from typing import Any

from django.templatetags.static import static

from drf_to_mkdoc.conf.settings import drf_to_mkdoc_settings
from drf_to_mkdoc.utils.common import extract_viewset_from_operation_id


class EndpointsIndexGenerator:
    def __init__(self, active_filters: list[str] | None = None):
        self.active_filters = set(
            active_filters
            or [
                "method",
                "path",
                "app",
                "models",
                "auth",
                "roles",
                "content_type",
                "params",
                "schema",
                "pagination",
                "ordering",
                "search",
                "tags",
            ]
        )

    def create_endpoint_card(
        self, endpoint: dict[str, Any], app_name: str, viewset_name: str
    ) -> str:
        method = endpoint["method"]
        path = endpoint["path"]
        filename = endpoint["filename"]
        view_class = extract_viewset_from_operation_id(endpoint["operation_id"])

        link_url = f"{app_name}/{viewset_name.lower()}/{filename}".replace(".md", "/index.html")
        data_attrs = f"""
            data-method="{method.lower()}"
            data-path="{path.lower()}"
            data-app="{app_name.lower()}"
            data-auth="{str(endpoint.get("auth_required", False)).lower()}"
            data-pagination="{str(endpoint.get("pagination_support", False)).lower()}"
            data-search="{str(bool(getattr(view_class, "search_fields", []))).lower()}"
            data-ordering="{str(endpoint.get("ordering_support", False)).lower()}"
            data-models="{" ".join(endpoint.get("related_models", [])).lower()}"
            data-roles="{" ".join(endpoint.get("permission_roles", [])).lower()}"
            data-content-type="{endpoint.get("content_type", "").lower()}"
            data-tags="{" ".join(endpoint.get("tags", [])).lower()}"
            data-schema="{" ".join(endpoint.get("schema_fields", [])).lower()}"
            data-params="{" ".join(endpoint.get("query_parameters", [])).lower()}"
        """.strip()

        return f"""
    <a href="{link_url}" class="endpoint-card" {data_attrs}>
        <span class="method-badge method-{method.lower()}">{method}</span>
        <span class="endpoint-path">{path}</span>
    </a>
    """

    def create_filter_section(self) -> str:
        filter_fields = {
            "method": """<div class="filter-group">
                <label class="filter-label">HTTP Method</label>
                <select id="filter-method" class="filter-select">
                    <option value="">All</option>
                    <option value="get">GET</option>
                    <option value="post">POST</option>
                    <option value="put">PUT</option>
                    <option value="patch">PATCH</option>
                    <option value="delete">DELETE</option>
                </select>
            </div>""",
            "path": """<div class="filter-group">
                <label class="filter-label">Endpoint Path</label>
                <input type="text" id="filter-path" class="filter-input"
                placeholder="Search path...">
            </div>""",
            "app": """<div class="filter-group">
                <label class="filter-label">Django App</label>
                <select id="filter-app" class="filter-select">
                    <option value="">All</option>
                    <!-- Dynamically filled -->
                </select>
            </div>""",
            "models": """<div class="filter-group">
                <label class="filter-label">Related Models</label>
                <input type="text" id="filter-models" class="filter-input">
            </div>""",
            "auth": """<div class="filter-group">
                <label class="filter-label">Authentication Required</label>
                <select id="filter-auth" class="filter-select">
                    <option value="">All</option>
                    <option value="true">Yes</option>
                    <option value="false">No</option>
                </select>
            </div>""",
            "roles": """<div class="filter-group">
                <label class="filter-label">Permission Roles</label>
                <input type="text" id="filter-roles" class="filter-input">
            </div>""",
            "content_type": """<div class="filter-group">
                <label class="filter-label">Content Type</label>
                <input type="text" id="filter-content-type" class="filter-input">
            </div>""",
            "params": """<div class="filter-group">
                <label class="filter-label">Query Parameters</label>
                <input type="text" id="filter-params" class="filter-input">
            </div>""",
            "schema": """<div class="filter-group">
                <label class="filter-label">Schema Fields</label>
                <input type="text" id="filter-schema" class="filter-input">
            </div>""",
            "pagination": """<div class="filter-group">
                <label class="filter-label">Pagination Support</label>
                <select id="filter-pagination" class="filter-select">
                    <option value="">All</option>
                    <option value="true">Yes</option>
                    <option value="false">No</option>
                </select>
            </div>""",
            "ordering": """<div class="filter-group">
                <label class="filter-label">Ordering Support</label>
                <select id="filter-ordering" class="filter-select">
                    <option value="">All</option>
                    <option value="true">Yes</option>
                    <option value="false">No</option>
                </select>
            </div>""",
            "search": """<div class="filter-group">
                <label class="filter-label">Search Support</label>
                <select id="filter-search" class="filter-select">
                    <option value="">All</option>
                    <option value="true">Yes</option>
                    <option value="false">No</option>
                </select>
            </div>""",
            "tags": """<div class="filter-group">
                <label class="filter-label">Tags</label>
                <input type="text" id="filter-tags" class="filter-input">
            </div>""",
        }

        fields_html = "\n".join(
            [html for key, html in filter_fields.items() if (key in self.active_filters)]
        )

        return f"""
        <div class="filter-sidebar collapsed" id="filterSidebar">
            <h3 class="filter-title">🔍 Filters</h3>
            <div class="filter-grid">
                {fields_html}
            </div>

            <div class="filter-actions">
                <button class="filter-apply" onclick="applyFilters()">Apply</button>
                <button class="filter-clear" onclick="clearFilters()">Clear</button>
            </div>

            <div class="filter-results">Showing 0 endpoints</div>
        </div>
        """

    def create_endpoints_index(
        self, endpoints_by_app: dict[str, list[dict[str, Any]]], docs_dir: Path
    ) -> None:
        stylesheets = [
            "stylesheets/endpoints/variables.css",
            "stylesheets/endpoints/base.css",
            "stylesheets/endpoints/theme-toggle.css",
            "stylesheets/endpoints/filter-section.css",
            "stylesheets/endpoints/layout.css",
            "stylesheets/endpoints/endpoints-grid.css",
            "stylesheets/endpoints/badges.css",
            "stylesheets/endpoints/endpoint-content.css",
            "stylesheets/endpoints/tags.css",
            "stylesheets/endpoints/sections.css",
            "stylesheets/endpoints/stats.css",
            "stylesheets/endpoints/loading.css",
            "stylesheets/endpoints/animations.css",
            "stylesheets/endpoints/responsive.css",
            "stylesheets/endpoints/accessibility.css",
            "stylesheets/endpoints/fixes.css",
        ]

        scripts = [
            "javascripts/endpoints-filter.js",
        ]
        prefix_path = f"{drf_to_mkdoc_settings.PROJECT_NAME}/"
        css_links = "\n".join(
            f'<link rel="stylesheet" href="{static(prefix_path + path)}">'
            for path in stylesheets
        )
        js_scripts = "\n".join(
            f'<script src="{static(prefix_path + path)}" defer></script>' for path in scripts
        )

        content = f"""# API Endpoints
<!-- inject CSS and JS directly -->
{css_links}
{js_scripts}

<div class="main-content">
        """
        content += self.create_filter_section()

        for app_name, endpoints in endpoints_by_app.items():
            content += f'<h2>{app_name.title()}</h2>\n<div class="endpoints-grid">\n'
            for endpoint in endpoints:
                viewset = endpoint["viewset"]
                content += self.create_endpoint_card(endpoint, app_name, viewset)
            content += "</div>\n"

        content += "</div>\n"
        output_path = docs_dir / "endpoints" / "index.md"
        output_path.parent.mkdir(parents=True, exist_ok=True)
        with Path(output_path).open("w", encoding="utf-8") as f:
            f.write(content)


def create_endpoints_index(
    endpoints_by_app: dict[str, list[dict[str, Any]]], docs_dir: Path
) -> None:
    generator = EndpointsIndexGenerator(
        active_filters=[
            "method",
            "path",
            "app",
            "search",
        ]
    )
    generator.create_endpoints_index(endpoints_by_app, docs_dir)



import importlib
import json
import re
from asyncio.log import logger
from functools import lru_cache
from pathlib import Path
from typing import Any

import yaml
from django.apps import apps
from django.core.exceptions import AppRegistryNotReady
from django.urls import resolve
from django.utils.module_loading import import_string
from drf_spectacular.generators import SchemaGenerator

from drf_to_mkdoc.conf.settings import drf_to_mkdoc_settings


class SchemaValidationError(Exception):
    """Custom exception for schema validation errors."""

    pass


class QueryParamTypeError(Exception):
    """Custom exception for query parameter type errors."""

    pass


def substitute_path_params(path: str, parameters: list[dict[str, Any]]) -> str:
    django_path = convert_to_django_path(path, parameters)

    django_path = re.compile(r"\{[^}]+\}").sub("1", django_path)
    django_path = re.sub(r"<int:[^>]+>", "1", django_path)
    django_path = re.sub(r"<uuid:[^>]+>", "12345678-1234-5678-9abc-123456789012", django_path)
    django_path = re.sub(r"<float:[^>]+>", "1.0", django_path)
    django_path = re.sub(r"<(?:string|str):[^>]+>", "dummy", django_path)
    django_path = re.sub(r"<path:[^>]+>", "dummy/path", django_path)
    django_path = re.sub(r"<[^:>]+>", "dummy", django_path)  # Catch remaining simple params

    return django_path  # noqa: RET504


def load_schema() -> dict[str, Any] | None:
    """Load the OpenAPI schema from doc-schema.yaml"""
    schema_file = Path(drf_to_mkdoc_settings.CONFIG_DIR) / "doc-schema.yaml"
    if not schema_file.exists():
        return None

    with schema_file.open(encoding="utf-8") as f:
        return yaml.safe_load(f)


def load_model_json_data() -> dict[str, Any] | None:
    """Load the JSON mapping data for model information"""
    json_file = Path(drf_to_mkdoc_settings.MODEL_DOCS_FILE)
    if not json_file.exists():
        return None

    with json_file.open("r", encoding="utf-8") as f:
        return json.load(f)


def load_doc_config() -> dict[str, Any] | None:
    """Load the documentation configuration file"""
    config_file = Path(drf_to_mkdoc_settings.DOC_CONFIG_FILE)
    if not config_file.exists():
        return None

    with config_file.open("r", encoding="utf-8") as f:
        return json.load(f)


def get_model_docstring(class_name: str) -> str | None:
    """Extract docstring from Django model class"""
    try:
        # Check if Django is properly initialized

        # Try to access Django apps to see if it's initialized
        apps.check_apps_ready()

        # Common Django app names to search
        app_names = drf_to_mkdoc_settings.DJANGO_APPS

        for app_name in app_names:
            try:
                # Try to import the models module
                models_module = importlib.import_module(f"{app_name}.models")

                # Check if the class exists in this module
                if hasattr(models_module, class_name):
                    model_class = getattr(models_module, class_name)

                    # Get the docstring
                    docstring = getattr(model_class, "__doc__", None)

                    if docstring:
                        # Clean up the docstring
                        docstring = docstring.strip()

                        # Filter out auto-generated or generic docstrings
                        if (
                            docstring
                            and not docstring.startswith(class_name + "(")
                            and not docstring.startswith("str(object=")
                            and not docstring.startswith("Return repr(self)")
                            and "django.db.models" not in docstring.lower()
                            and len(docstring) > 10
                        ):  # Minimum meaningful length
                            return docstring

            except (ImportError, AttributeError):
                continue

    except (ImportError, AppRegistryNotReady):
        # Django not initialized or not available - skip docstring extraction
        pass

    return None


def extract_app_from_operation_id(operation_id: str) -> str:
    view = extract_viewset_from_operation_id(operation_id)

    if isinstance(view, type):
        module = view.__module__
    elif hasattr(view, "__class__"):
        module = view.__class__.__module__
    else:
        raise TypeError("Expected a view class or instance")

    return module.split(".")[0]


@lru_cache
def get_custom_schema():
    custom_schema_path = Path(drf_to_mkdoc_settings.CUSTOM_SCHEMA_FILE)
    if not custom_schema_path.exists():
        return {}

    try:
        with custom_schema_path.open(encoding="utf-8") as file:
            data = json.load(file)
    except Exception:
        return {}

    for _operation_id, overrides in data.items():
        parameters = overrides.get("parameters", [])
        if not parameters:
            continue
        for parameter in parameters:
            if {"name", "in", "description", "required", "schema"} - set(parameter.keys()):
                raise SchemaValidationError("Required keys are not passed")

            if parameter["in"] == "query":
                queryparam_type = parameter.get("queryparam_type")
                if not queryparam_type:
                    raise QueryParamTypeError("queryparam_type is required for query")

                if queryparam_type not in (
                    {
                        "search_fields",
                        "filter_fields",
                        "ordering_fields",
                        "filter_backends",
                        "pagination_fields",
                    }
                ):
                    raise QueryParamTypeError("Invalid queryparam_type")
    return data


def convert_to_django_path(path: str, parameters: list[dict[str, Any]]) -> str:
    """
    Convert a path with {param} to a Django-style path with <type:param>.
    If PATH_PARAM_SUBSTITUTE_FUNCTION is set, use that function instead.
    """
    function = None
    func_path = drf_to_mkdoc_settings.PATH_PARAM_SUBSTITUTE_FUNCTION

    if func_path:
        try:
            function = import_string(func_path)
        except ImportError:
            logger.warning("PATH_PARAM_SUBSTITUTE_FUNCTION is not a valid import path")

    # If custom function exists and returns a valid value, use it
    PATH_PARAM_SUBSTITUTE_MAPPING = drf_to_mkdoc_settings.PATH_PARAM_SUBSTITUTE_MAPPING
    if callable(function):
        try:
            result = function(path, parameters)
            if result and isinstance(result, dict):
                PATH_PARAM_SUBSTITUTE_MAPPING.update(result)
        except Exception as e:
            logger.exception("Error in custom path substitutor: %s", e)

    # Default Django path conversion
    def replacement(match):
        param_name = match.group(1)
        custom_param_type = PATH_PARAM_SUBSTITUTE_MAPPING.get(param_name)
        if custom_param_type and custom_param_type in ("int", "uuid", "str"):
            converter = custom_param_type
        else:
            param_info = next((p for p in parameters if p.get("name") == param_name), {})
            param_type = param_info.get("schema", {}).get("type")
            param_format = param_info.get("schema", {}).get("format")

            if param_type == "integer":
                converter = "int"
            elif param_type == "string" and param_format == "uuid":
                converter = "uuid"
            else:
                converter = "str"

        return f"<{converter}:{param_name}>"

    return re.sub(r"{(\w+)}", replacement, path)


@lru_cache
def get_schema():
    base_schema = SchemaGenerator().get_schema(request=None, public=True)

    custom_data = get_custom_schema()
    if not custom_data:
        return base_schema

    # Map operation_id → (path, method)
    op_map = {}
    for path, actions in base_schema.get("paths", {}).items():
        for method, op_data in actions.items():
            operation_id = op_data.get("operationId")
            if operation_id:
                op_map[operation_id] = (path, method)

    allowed_keys = {"description", "parameters", "requestBody", "responses"}
    for operation_id, overrides in custom_data.items():
        if operation_id not in op_map:
            continue

        append_fields = set(overrides.get("append_fields", []))
        path, method = op_map[operation_id]
        target_schema = base_schema["paths"][path][method]
        for key in allowed_keys:
            if key not in overrides:
                continue

            custom_value = overrides[key]
            base_value = target_schema.get(key)

            if key in append_fields and isinstance(base_value, list):
                target_schema[key].extend(list(custom_value))
            else:
                #  Otherwise, replace
                target_schema[key] = custom_value

    return base_schema


@lru_cache
def get_operation_id_path_map() -> dict[str, str]:
    schema = get_schema()
    paths = schema.get("paths", {})
    mapping = {}

    for path, actions in paths.items():
        for _http_method_name, action_data in actions.items():
            operation_id = action_data.get("operationId")
            if operation_id:
                mapping[operation_id] = path, action_data.get("parameters", [])

    return mapping


def extract_viewset_from_operation_id(operation_id: str):
    """Extract the ViewSet class from an OpenAPI operation ID."""
    operation_map = get_operation_id_path_map()
    path, parameters = operation_map.get(operation_id)

    if not path:
        raise ValueError(f"Path not found for operation ID: {operation_id}")

    resolved_path = substitute_path_params(path, parameters)
    try:
        match = resolve(resolved_path)
        view_func = match.func
        if hasattr(view_func, "view_class"):
            # For generic class-based views
            return view_func.view_class
        try:
            # For viewsets
            return view_func.cls
        except AttributeError:
            pass
        else:
            return view_func

    except Exception:
        logger.error(
            f"Failed to resolve path.\nschema_path{path}\ntried_path={resolved_path}\n---"
        )


def extract_viewset_name_from_operation_id(operation_id: str):
    view_cls = extract_viewset_from_operation_id(operation_id)
    return view_cls.__name__ if hasattr(view_cls, "__name__") else str(view_cls)


def format_method_badge(method: str) -> str:
    """Create a colored badge for HTTP method"""
    return f'<span class="method-badge method-{method.lower()}">{method.upper()}</span>'


def write_file(file_path: str, content: str) -> None:
    full_path = Path(drf_to_mkdoc_settings.DOCS_DIR) / file_path
    full_path.parent.mkdir(parents=True, exist_ok=True)
    with full_path.open("w", encoding="utf-8") as f:
        f.write(content)


def get_model_description(class_name: str) -> str:
    """Get a brief description for a model with priority-based selection"""
    # Priority 1: Description from config file
    config = load_doc_config()
    if config and "model_descriptions" in config:
        config_description = config["model_descriptions"].get(class_name, "").strip()
        if config_description:
            return config_description

    # Priority 2: Extract docstring from model class
    docstring = get_model_docstring(class_name)
    if docstring:
        return docstring

    # Priority 3: static value
    return "Not provided"


def get_app_descriptions() -> dict[str, str]:
    """Get descriptions for Django apps from config file"""
    config = load_doc_config()
    if config and "app_descriptions" in config:
        return config["app_descriptions"]

    # Fallback to empty dict if config not available
    return {}


def create_safe_filename(path: str, method: str) -> str:
    """Create a safe filename from path and method"""
    safe_path = re.sub(r"[^a-zA-Z0-9_-]", "_", path.strip("/"))
    return f"{method.lower()}_{safe_path}.md"
